package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
/*
 * Clase encarga de seleccional el audio en entrada del programa
 */

public class AudioEntrada implements OutSignal, UpdateListener {
    
    private final int id;
    private static int nextId=0;
    private ArrayList<InSignal> drains;
    private AudioDispatcher dispatcher;
    private AudioFormat audioFormat;
    private Simulator sim;
    private File AudioFile;
    
    {  id = nextId++;
    }
    /**
     * Constructor, recive como parametro el entorno de simulacion
     * @param simu 
     */
    public AudioEntrada(Simulator simu){
        sim = simu;
        drains = new ArrayList<>();
        dispatcher = OpenAudio();
        //escuchar();
        sim.addEntrada(this);
    }
    
    /**
     * Metodo Encargado de abrir el audio
     */
    
    private AudioDispatcher OpenAudio(){
        
        AudioInputStream audioInputStream;
        SourceDataLine line;
        
        //Abriendo ventana para eleccion de archivo (JFileChooser)
        if (id!= 0){
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        audioFormat = null;
        //Tratando de Abrir el Archivo
        try
        {   
            int returnVal = 0;
            while(AudioFile==null){
                returnVal = chooser.showOpenDialog(null);
                AudioFile = chooser.getSelectedFile();
            }
            if(returnVal == JFileChooser.APPROVE_OPTION) 
                System.out.println("Audio Seleccionado : " + AudioFile.getName());
            audioInputStream = AudioSystem.getAudioInputStream(AudioFile);
            audioFormat = audioInputStream.getFormat();
            dispatcher = new AudioDispatcher(audioInputStream, audioFormat.getFrameSize(),0); 
        }
        catch (UnsupportedAudioFileException | IOException e)
        {
            e.printStackTrace();
            System.exit(1);
        }
        }
        return dispatcher;
    }

    public void propagateOutputValue() {
        for (InSignal drain: drains) 
            drain.inputValueChanged();
    }
    
    public AudioDispatcher getDispatcher() {
        return dispatcher;
    }
    
    public void connectOutput(InSignal drain) {
        if(drain!=null) {
          drains.add(drain);
          drain.addSource(this);
        }
    }

    public void connectDrain(InSignal drain) {
        if(drain!=null) drains.add(drain);
    }
    
    public void removeFromDrains(InSignal drain) {
        if(drain!=null) drains.remove(drain);
    }
    
    public void disconnectAllOutput() {
        while (!drains.isEmpty()) {
            drains.get(0).removeFromSource(this);
            drains.remove(0);
        }
    }
    
    public String getDescription() {
        return getClass().getName()+"_"+id;
    }
    
    /**
     * Retorna el formato del aduio de entrada
     */
    public AudioFormat getFormat(){
        return audioFormat;
    }
    
    /**
     * Reinicia la entrada para la resimulacion del circuito
     */
    public void restart(){
        try {
            AudioInputStream audioInputStream;
            SourceDataLine line;
            audioInputStream = AudioSystem.getAudioInputStream(AudioFile);
            audioFormat = audioInputStream.getFormat();
            dispatcher = new AudioDispatcher(audioInputStream, audioFormat.getFrameSize(),0);
        } catch (UnsupportedAudioFileException | IOException ex) {
            Logger.getLogger(AudioEntrada.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
}
