package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import be.hogent.tarsos.dsp.GainProcessor;
import be.hogent.tarsos.dsp.effects.DelayEffect;
import javax.swing.JPanel;

/**
 *Clase encargada de Implementar la funcionalidad porpias del efecto echo-delay
 */
public class Delay extends Bloque {
    
    private OutSignal connected;
    private static int nextId = 0;
    private AudioDispatcher dispatcher;
    private DelayEffect delayEffect;
    private double Gain;//%
    private double Delay;//ms
    private double Decay;//%
    private final double defaultInputGain = 100;//%
    private final double defaultDelay = 200;//ms
    private final double defaultDecay = 50;//%
    private GainProcessor inputGain;
    private PDelay panel;
    /**
     * Constructor
     */
    public Delay(){
        super(nextId++);
        connected = null;
        dispatcher = null;
        panel = new PDelay(this);
        Gain = defaultInputGain;//%
        Delay = defaultDelay;//ms
        Decay = defaultDecay;//%
    } 
    public void connectInput(OutSignal output){
        connected = output;
        output.connectDrain(this);
    }
    
    public void disconnectInput(){
        while(connected != null) {
            connected.removeFromDrains(this);
            connected = null;
        }
    }
    
    public void addSource(OutSignal output) {
        connected = output;
    }
    
    public void removeFromSource(OutSignal output){
        connected = null;
    }
    
    public void computeOutput(){
        delayEffect = new DelayEffect(Delay/1000,Decay/100.0,dispatcher.getFormat().getSampleRate());
        inputGain = new GainProcessor(Gain/100.0);
        System.out.println("Delay="+Delay+" Decay="+Decay+" Gain="+Gain);
        dispatcher.addAudioProcessor(inputGain);
        dispatcher.addAudioProcessor(delayEffect);
        this.propagateOutputValue();
    }
    
    public boolean hasConnectedSource() {
        return (connected != null);
    }
    
    public AudioDispatcher getDispatcher(){
        return dispatcher;
    }
    
    public void inputValueChanged() {
         dispatcher = connected.getDispatcher();
         this.computeOutput();
         this.propagateOutputValue();
    }
    /**
     * Metodo encargado de setear el parametro de ganancia del efecto
     * @param newgain 
     */
    public void setGain(double newgain){
        Gain = newgain*100;
    }
    /**
     * Metodo encargado de setear el parametro de tiempo de repeticion del echo
     * @param newecho 
     */
    public void setEchoLength(double newecho){
        Delay = newecho*100;
    }
    
    /**
     * Metodo encargado de setear el parametro de decaimiento del bloque
     * @param newdelay 
     */
    public void setDecay(double newdelay){
        Decay = newdelay*100;
    }
    
    public JPanel getPanel(){
        return panel;
    }
    
}
