package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import be.hogent.tarsos.dsp.filters.HighPass;
import be.hogent.tarsos.dsp.filters.LowPassFS;
import be.hogent.tarsos.dsp.util.AudioFloatConverter;
import java.util.ArrayList;
import javax.swing.JPanel;

/**
 * Clase Encargada de Ejecutar las funciones de un filtro digital pasa banda.
 */

public class FiltroPasabanda extends Bloque{

    private ArrayList<OutSignal> sources;
    private static int nextId = 0;
    private AudioDispatcher dispatcher;
    private int FreqAlta = 3000;
    private int FreqBaja = 500;
    private PPasabanda panel;
    /**
     * Constructor
     */
    public FiltroPasabanda(){
        super(nextId++);    
        sources= new ArrayList<OutSignal>();
        panel = new PPasabanda(this);
    }

      public void connectInput(OutSignal output){
        sources.add(output);
        output.connectDrain(this);
    }
   
    public void disconnectInput(){
        while(!sources.isEmpty()) {
            sources.get(0).removeFromDrains(this);
            sources.remove(0);
        }
    }
    
    public void addSource(OutSignal output) {
        sources.add(output);
    }
    
    public void removeFromSource(OutSignal output){
        sources.remove(output);
    }
    
    /**
     * Verifica si tiene conectada alguna entrada
     * @return 
     */
    public boolean hasConnectedSource() {
        return sources.size()>0;
    }

    @Override
    public void computeOutput() {
        dispatcher.addAudioProcessor(new LowPassFS(FreqAlta/10, dispatcher.getFormat().getSampleRate()));
        dispatcher.addAudioProcessor(new HighPass(FreqBaja/10, dispatcher.getFormat().getSampleRate()));
        this.propagateOutputValue();
    }

    @Override
    public AudioDispatcher getDispatcher() {
        return dispatcher;
    }

    @Override
    public void inputValueChanged() {
         dispatcher = sources.get(0).getDispatcher();
         this.computeOutput();
         this.propagateOutputValue();
    }

    @Override
    public JPanel getPanel() {
        return panel;
    }
    
    /**
     * Metodo encargado de setear el parametro correspondiente a la frecuencia superior del filtro
     * @param newfreq 
     */
    public void setFreqAlta(int newfreq){
        FreqAlta = newfreq;
    }  
    
    /**
     * Metodo encargado de setear el parametro correspondiente a la frecuencua inferior del filtro
     * @param newfreq 
     */
    public void setFreqBaja(int newfreq){
        FreqBaja = newfreq;
    }
}
