package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import be.hogent.tarsos.dsp.GainProcessor;
import be.hogent.tarsos.dsp.effects.FlangerEffect;
import java.util.ArrayList;
import javax.swing.JPanel;

public class Flanger extends Bloque {

    private ArrayList<OutSignal> sources;
    private static int nextId = 0;
    private FlangerEffect flangerEffect;
    private GainProcessor inputGain;
    private int defaultInputGain = 100;//%
    private int defaultLength = 20;//ms
    private int defaultImpact = 50;//%
    private int defaultFrequency = 10;//Hz
    private AudioDispatcher dispatcher;    
    private PFlanger panel;
    
    /**
     * Constructor
     */
    public Flanger(){
        super(nextId++);
        sources= new ArrayList<OutSignal>();
        panel = new PFlanger(this);
    }

    public void connectInput(OutSignal output){
        sources.add(output);
        output.connectDrain(this);
    }
   
    public void disconnectInput(){
        while(!sources.isEmpty()) {
            sources.get(0).removeFromDrains(this);
            sources.remove(0);
        }
    }
    
    public void addSource(OutSignal output) {
        sources.add(output);
    }
    
    public void removeFromSource(OutSignal output){
        sources.remove(output);
    }
    
    /**
     * Metodo encargado de verificar si tiene alguna fuente conectada
     * @return 
     */
    public boolean hasConnectedSource() {
        return sources.size()>0;
    }

    public void computeOutput() {
        flangerEffect = new FlangerEffect(defaultLength/1000.0,defaultImpact/100.0,dispatcher.getFormat().getSampleRate(),defaultFrequency/10.0);			
	inputGain = new GainProcessor(defaultInputGain/100.0);
        dispatcher.addAudioProcessor(inputGain);
        dispatcher.addAudioProcessor(flangerEffect);
        this.propagateOutputValue();
    }

    public AudioDispatcher getDispatcher() {
        return dispatcher;
    }

    public void inputValueChanged() {
         dispatcher = sources.get(0).getDispatcher();
         this.computeOutput();
         this.propagateOutputValue();
    }
    
    public JPanel getPanel(){
        return panel;
    }
    
    /**
     * Metodo encargado de setear el parametro correspondiente a la ganancia
     * @param newgain 
     */
    public void setGain(int newgain){
        defaultInputGain = newgain;
    }
    
    /**
     * Metodo encargado de setear el parametro correspondiente a lo cargado que sea el efecto
     * @param newimpact 
     */
    public void setImpact(int newimpact){
        defaultImpact = newimpact;
    }
    
    /**
     * Metodo encargado de setear el parametro de la frecuencia asociada al efecto flanger
     * @param newfrequency 
     */
    public void setFrequency(int newfrequency){
        defaultFrequency = newfrequency;
    }
}
