package audiotesting;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.font.GlyphVector;
import java.awt.geom.*;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

/**
 * Clase encargada de generar el entorno grafico de la entrada de audio
 * @author dcardenasn
 */
public class GAudioEntrada extends AudioEntrada implements Drawable, GOutSignal{
   
    private static final Path2D.Double polyline = 
                      new Path2D.Double(Path2D.WIND_EVEN_ODD,5);
    private static final Rectangle2D.Double output= new Rectangle2D.Double(75,30,10,10);
    static {  // static initialization block. 
        polyline.moveTo (60, 35);
        polyline.lineTo(75, 35);
    }
    private Color color = Color.BLACK;
    private Path2D.Double shape;
    private Path2D.Double out;
    private static BasicStroke stroke;
    private BufferedImage image;
    private Path2D.Double imageshape;
    
    /**
     * Constructor
     * @param pos
     * @param sim 
     */
    public GAudioEntrada(Point2D.Double pos, Simulator sim){
        // Constructor padre
        super(sim);
        // Construllendo la imagen a colocar en la simulacion
        openimage();
        AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
        shape = (Path2D.Double) at.createTransformedShape(polyline);
        out = (Path2D.Double) at.createTransformedShape(output);
        stroke = new BasicStroke(2);
        imageshape = (Path2D.Double) at.createTransformedShape(
                new Rectangle2D.Double(0,0,60,70));
        
    }
    
    public void draw (Graphics2D g){
       GlyphVector shapetext;
       Font font = new Font("monospaced", Font.BOLD, 15);
       String[] name = this.getDescription().split("_");
       shapetext = font.createGlyphVector(g.getFontRenderContext(),"#"+name[1]);
       
       g.setColor(Color.BLACK);
       g.setStroke(stroke);
       g.draw(shape);
       g.setColor(color);
       g.setStroke(stroke);
       g.drawImage(image, (int) imageshape.getBounds2D().getX()-10, (int) imageshape.getBounds2D().getY()-5, null);
       //g.draw(imageshape);
       g.draw(out);
       g.drawGlyphVector(shapetext, (float) imageshape.getBounds2D().getX()+10, (float) imageshape.getBounds2D().getY()+20);
    }
    
    public boolean contains (Point p){
        return imageshape.contains(p);
    }

    public void dragIn(int dx, int dy) {
       AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
       imageshape.transform(at);
       shape.transform(at);
       out.transform(at);
    }
    
    public void flip() {
        AffineTransform af = new AffineTransform();   
        Rectangle2D box = imageshape.getBounds2D(); 
        double x = box.getX()+box.getWidth()/2;

        // Note: the transformations get applied IN REVERSE ORDER!
        af.translate(x,0);
        af.scale(-1, 1);
        af.translate(-x, 0);          
        shape.transform(af);
        out.transform(af);
        
        AffineTransform tx = AffineTransform.getScaleInstance(-1, 1);
        tx.translate(-image.getWidth(null), 0);
        AffineTransformOp op = new AffineTransformOp(tx, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
        image = op.filter(image, null);
    }
    
    public void setSelected() {
        color = Color.RED;
    }

    public void setReleased(){
        color = Color.BLACK;
    }
    
    public Shape getGOutput() {
        return out;
    }
    
    public Shape getGInput() {
     return null;
   }
   
    public void openimage(){
        try {
            image = ImageIO.read(new File("./Images/audio80x80v2.png"));
        } catch (IOException e) {
                e.printStackTrace();
        }
   }
    
   public Bloque getBloque(){
       return null;
   }
   
    public void removeDraw(){
        disconnectAllOutput();
    }
   
}
