package audiotesting;

import java.awt.*;
import java.awt.font.GlyphVector;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

/*
 * Clase encargada de definir la configuracion grafica del bloque Delay
 */

public class GDelay extends Delay implements Drawable, GOutSignal, GInSignal{
    
    private static final Path2D.Double polyline = new Path2D.Double(Path2D.WIND_EVEN_ODD,10);
    private static final Rectangle2D.Double input= new Rectangle2D.Double(-5,15,10,10);
    private static final Rectangle2D.Double output= new Rectangle2D.Double(115,15,15,10);
    private Color color = Color.BLACK;
    private Path2D.Double shape;
    private Path2D.Double in;
    private Path2D.Double out;
    private final BasicStroke stroke;
    private BufferedImage image;
    private Path2D.Double imageshape;
    
    static {
        //Linea in
        polyline.moveTo(0,20);
        polyline.lineTo(20,20);
        //Linea out
        polyline.moveTo(100,20);
        polyline.lineTo(120,20);
   }
    
    /**
     * Constructor, recive como entrada el lugar inicial en donde se coloca la compuerta
     * @param pos 
     */
    public GDelay(Point2D.Double pos){
        super(); 
        openimage();
        AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
        shape = (Path2D.Double) at.createTransformedShape(polyline);
        in = (Path2D.Double) at.createTransformedShape(input);
        out = (Path2D.Double) at.createTransformedShape(output);
        stroke = new BasicStroke(4);
        imageshape = (Path2D.Double) at.createTransformedShape(
                new Rectangle2D.Double(20,0,80,50));
    }

    public void draw (Graphics2D g){
      GlyphVector shapetext;
      Font font = new Font("monospaced", Font.BOLD, 15);
      String[] name = this.getDescription().split("_");
      shapetext = font.createGlyphVector(g.getFontRenderContext(),"#"+name[1]);
      g.drawImage(image, (int) imageshape.getBounds2D().getX()+0, (int) imageshape.getBounds2D().getY()-20, null);
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(shape);
      g.setStroke(new BasicStroke(2));
      if (hasConnectedSource())
        g.fill(in);
      else
        g.draw(in);
      g.draw(out);
      g.setColor(Color.WHITE);
      g.drawGlyphVector(shapetext, (float) shape.getBounds2D().getX()+75, (float) shape.getBounds2D().getY()+30);
   }
   
   public boolean contains (Point p){
     return imageshape.contains(p);
   }
   
   public void dragIn(int dx, int dy) {
      AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
      shape.transform(at);
      imageshape.transform(at);
      in.transform(at);
      out.transform(at);
      disconnectInput();
      disconnectAllOutput();
   }
   
   public void flip() {
     AffineTransform af = new AffineTransform();   
     Rectangle2D box = shape.getBounds2D(); 
     double x = box.getX()+box.getWidth()/2;

     // Note: the transformations get applied IN REVERSE ORDER!
     af.translate(x,0);
     af.scale(-1, 1);
     af.translate(-x, 0);  
    
     shape.transform(af);
     in.transform(af);
     out.transform(af);
   }
   
   public void setSelected() {
     color = Color.RED;
   }
   
   public void setReleased(){
     color = Color.BLACK;
   }
   
   public Shape getGOutput() {
     return out;
   }
   
   public Shape getGInput() {
     return in;
   }
   
   public Bloque getBloque(){
       return this;
   }
   
    public void removeDraw(){
        disconnectInput();
        disconnectAllOutput();
    }
    
    public void openimage(){
        try {
            image = ImageIO.read(new File("./Images/Echo.png"));
        } catch (IOException e) {
                e.printStackTrace();
        }
    }
   
}