package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import java.util.*;

/**
 *Metodo encargado de asociar la propiedades de la coneccion de bloques
 */
public class Wire implements InSignal, OutSignal {
  private OutSignal source;
  private ArrayList <InSignal> drains;
  
  public Wire() {
    source = null;
    drains = new ArrayList<InSignal>();
  }
  // InSignal interface
  public void inputValueChanged() {
    for ( InSignal s: drains )
       s.inputValueChanged(); 
  }
  
  public void connectInput(OutSignal output) {
    source = output;
    output.connectDrain(this);
  }
  
  public void disconnectInput() {
    if (source!=null) {
      source.removeFromDrains(this);
      source = null;
    }
  }
  
  public void addSource(OutSignal output) {
    source=output;
  }
  
  public void removeFromSource(OutSignal output) {
    if (source==output) source=null;
  }// InSignal Interface end
  
  // OutSignal interface
  public AudioDispatcher getDispatcher() {
    return source.getDispatcher(); 
  }
  
  public void connectOutput(InSignal drain){
    if (drain != null ) {
      drains.add(drain);
      drain.addSource(this);
    }
  }
  
  public void connectDrain(InSignal drain) {
    if(drain!=null) drains.add(drain);
  }
  
  public void removeFromDrains(InSignal drain) {
    drains.remove(drain);
  }
  
  /**
   * desconecta la salida y la desliga de la clase
   * @param drain 
   */
  public void disconnectOutput(InSignal drain){
    if (drain != null) {
      drains.remove(drain);
      drain.removeFromSource(this);
    }
  }
  
  public void disconnectAllOutput(){
    while(drains.size()!=0) {
      drains.get(0).removeFromSource(this);
      drains.remove(0);
    }
  }
  
  public String getDescription() {
    if (source!=null) return source.getDescription();
    else return ("Nothing");
  }
  
  /**
   * Verifica si tiene una salida conectada
   * @return 
   */
  public boolean hasConnectedSource(){
    return source != null;
  }
} 